dnl $Header: /cvsroot/silky/silky/macros/silc.m4,v 1.5 2003/12/13 01:40:16 toniw Exp $
dnl 
dnl Copyright (C) 2003 Matthew Aldous <Matthew@Aldous.com>
dnl 
dnl Redistribution and use in source and binary forms, with or without
dnl modification, are permitted provided that the following conditions
dnl are met:
dnl 
dnl  1. Redistributions of source code must retain the above copyright
dnl notice, this list of conditions and the following disclaimer.
dnl 
dnl  2. Redistributions in binary form must reproduce the above copyright
dnl notice, this list of conditions and the following disclaimer in
dnl the documentation and/or other materials provided with the
dnl distribution.
dnl 
dnl  3. The name of the author may not be used to endorse or promote
dnl products derived from this software without specific prior written
dnl permission.
dnl 
dnl THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS
dnl OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
dnl WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
dnl ARE DISCLAIMED. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
dnl DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
dnl DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
dnl GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
dnl INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER
dnl IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
dnl OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
dnl IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

AC_DEFUN(AM_WITH_SILC_INCLUDES,
	[AC_ARG_WITH(silc-includes,
	 AC_HELP_STRING([--with-silc-includes],
	                [with silc (default is /usr/local/silc/include)]),
	, 
	with_silc_includes=yes)
	
	if test x"$with_silc_includes" != x"yes"; then
		SILC_INCLUDES_PREFIX="$with_silc_includes"
	else
		SILC_INCLUDES_PREFIX="/usr/local/silc/include"
	fi
	
	SILC_CFLAGS="-I$SILC_INCLUDES_PREFIX $INCINTL"
	SILC_CPPFLAGS="-I$SILC_INCLUDES_PREFIX $INCINTL"
					  
	AC_SUBST(SILC_INCLUDES_PREFIX)	
	AC_SUBST(SILC_CFLAGS)
	AC_SUBST(SILC_CPPFLAGS)

])

AC_DEFUN(AM_WITH_SILC_LIBS,
	[AC_ARG_WITH(silc-libs,
	 AC_HELP_STRING([--with-silc-libs],
	                [with silc (default is /usr/local/silc/lib)]),
	, 
	with_silc_libs=yes)
	
	if test x"$with_silc_libs" = x"no"; then
		AC_MSG_ERROR([silc=no])
	else
		AC_CACHE_CHECK([if compiler recognizes -pthread],
				silc_cv_gcc_pthread,
				ac_save_CFLAGS=$CFLAGS
				CFLAGS="$CFLAGS -pthread"
				AC_LANG_SAVE
				AC_LANG_C
				AC_TRY_LINK([#include <pthread.h>],
					    [void *p = pthread_create;],
					    silc_cv_gcc_pthread=yes,
					    silc_cv_gcc_pthread=no
				)
				AC_LANG_RESTORE
				CFLAGS=$ac_save_CFLAGS
		)

		if test x"$silc_cv_gcc_pthreadx" = x"yes"; then
			SILC_LIBS=""
			CFLAGS="$CFLAGS -pthread"
		else
			AC_CHECK_LIB(pthread,pthread_create,SILC_LIBS=-lpthread,
			[AC_CHECK_LIB(c_r,pthread_create,SILC_LIBS=-lc_r)])
		fi

		_SILC_CFLAGS="$CFLAGS"
		_SILC_CPPFLAGS="$CPPFLAGS"
		_SILC_LIBS="$LIBS"
	
		SILC_PREFIX=""
	
		if test x"$with_silc_libs" != x"yes"; then
			SILC_PREFIX="$with_silc_libs"
		else
			SILC_PREFIX="/usr/local/silc/lib"
		fi
		
		case "$host" in
			*mingw* )
			SILC_CFLAGS=""
			SILC_CPPFLAGS=""
			SILC_LIBS="$SILC_LIBS -lwsock32"
			;;
		esac

		if test x"$gt_cv_func_gnugettext1_libc" = x"yes"; then
			SILC_CFLAGS="$SILC_CFLAGS -I$SILC_INCLUDES_PREFIX"
			SILC_CPPFLAGS="$SILC_CPPFLAGS -I$SILC_INCLUDES_PREFIX"
			SILC_LIBS="$SILC_LIBS -L$SILC_PREFIX -lsilcclient -lsilc"
		else
			SILC_CFLAGS="$SILC_CFLAGS -I$SILC_INCLUDES_PREFIX $INCINTL"
			SILC_CPPFLAGS="$SILC_CPPFLAGS -I$SILC_INCLUDES_PREFIX $INCINTL"
			SILC_LIBS="$SILC_LIBS -L$SILC_PREFIX -lsilcclient -lsilc $LIBINTL"
		fi

		if test x"$am_cv_lib_iconv" = x"yes"; then
			SILC_CFLAGS="$SILC_CFLAGS $INCICONV"
			SILC_CPPFLAGS="$SILC_CPPFLAGS $INCICONV"
			SILC_LIBS="$SILC_LIBS $LIBICONV"
		fi

		CFLAGS="$CFLAGS $SILC_CFLAGS"
		CPPFLAGS="$CPPFLAGS $SILC_CPPFLAGS"
		LIBS="$LIBS $SILC_LIBS"

		AC_CHECK_HEADERS(silcincludes.h, 
				 , 
				 silc_includes_found=no)
		
		if test x"$silc_includes_found" = x"no"; then
			AC_MSG_ERROR([Could not find silc includes from $SILC_INCLUDES_PREFIX])
		fi
		
		if test x"$am_cv_lib_iconv" = x"yes"; then
			AC_CHECK_LIB(silcclient, 
				     silc_client_run_one,
				     [SILC_CFLAGS="-I$SILC_INCLUDES_PREFIX $INCICONV $INCINTL"
				      SILC_CPPFLAGS="-I$SILC_INCLUDES_PREFIX $INCICONV $INCINTL"
				      SILC_LIBS="-L$SILC_PREFIX -lsilcclient -lsilc $LIBICONV $LIBINTL"
		       		      silc_found=yes],
		       		      [silc_found=no],
		       		      "$SILC_LIBS")
		else
			AC_CHECK_LIB(silcclient, 
				     silc_client_run_one,
				     [SILC_CFLAGS="-I$SILC_INCLUDES_PREFIX $INCINTL"
				      SILC_CPPFLAGS="-I$SILC_INCLUDES_PREFIX $INCINTL"
				      SILC_LIBS="-L$SILC_PREFIX -lsilcclient -lsilc $LIBINTL"
		       		      silc_found=yes],
		       		      [silc_found=no],
		       		      "$SILC_LIBS")
		fi
		
		if test x"$silc_found" = x"no"; then
			AC_MSG_ERROR([Could not find silc libraries from $SILC_PREFIX])
		fi
		
		CFLAGS="$_SILC_CFLAGS"
		CPPFLAGS="$_SILC_CPPFLAGS"
		LIBS="$_SILC_LIBS"
		
		AC_SUBST(SILC_PREFIX)
		AC_SUBST(SILC_CFLAGS)
		AC_SUBST(SILC_CPPFLAGS)
		AC_SUBST(SILC_LIBS)
	fi
])
